/*****************************************************************************
 *                                                                           *
 *                                 MQSAMPLE.C                                *
 *                                                                           *
 *                     LAN Distributed Platform - LANDP                      *
 *                                                                           *
 *                5639-I90 (C) Copyright IBM Corp 2000                       *
 *        All Rights Reserved - Licensed Materials - Property of IBM         *
 *                                                                           *
 *****************************************************************************
 *                                                                           *
 * This program is provided in the hope that it will be useful, but comes    *
 * with no guarantees whatsoever and no responsibility or liability is       *
 * accepted by the author or by IBM Corp for any loss or consequential loss  *
 * that may be suffered by the use (or otherwise) of this program or of any  *
 * program derived from it. Comments elsewhere in this program must not be   *
 * interpretted as varying the statement in this paragraph.                  *
 *                                                                           *
 *****************************************************************************


This sample client program illustrates the use of the LANDP MQSeries Link server.

A number of configuration steps must have been completed before this sample code
will work :-

1   Install and configure MQSeries Server on a workstation in the LANDP workgroup
    so that there is a working MQSeries Queue Manager (MQM).

2   Install and customize LANDP so that the LANDP MQSeries Link server is on the
    same workstation as the MQM. As part of the LANDP customization define at 
    least one MQ Link server client definition for use by this sample.

3   Define the MQSeries Queues that are to be used with this sample.

--------------------------------------------------------------------------------

The following routines show how LANDP client programs can be written to use the
API calls of the LANDP MQSeries Link server.

The sample provides a collection of interactive functions designed to show the
typical use of each of the API calls. The functions can be run individually
via the interface to build up a more complex interaction with LANDP in a step by
step manner.  All the parameters used in the Interactive Tests can be changed
(all the names start glb_), and the parameter retains the new value for the rest
of the test.

The routines use the following API calls :

setup_parameters ()         - Allow user to inspect and change PARMLIST parameters
begin_transaction ()        - Begin Transaction (BT Function)
put_to_q ()                 - MQPUT (PQ Function)
get_from_q ()               - MQGET (GQ Function)
browse_from_q ()            - MQGET (GQ Function) with Browse Message Options
rollback_transaction ()     - Rollback (RB Function)
commit_transaction ()       - Checkpoint (CP Function)
end_transaction ()          - End Transaction (ET Function)
close_session ()            - Close Session (CS Function)


The parameters are defined as follows :

glb_server              - The MQSeries Link server name
glb_queue_name          - The MQ Queue Name to be used in the functions.
glb_message_id          - The Message Id used in GQ and PQ functions.
glb_correlation_id      - The Correlation Id used in GQ and PQ functions.
glb_user_id             - The User Id used in GQ and PQ functions.
glb_wait_interval       - The Wait Interval used in GQ functions.
glb_GQ_message_opts     - GQ Message Options (see LANDP PROGRAMMING REFERENCE for details)
glb_BR_message_opts     - GQ Message Options for Browse (see LANDP PROGRAMMING REFERENCE)
glb_message_type        - PQ Message Type (see LANDP PROGRAMMING REFERENCE for details)
glb_reply_queue_name    - The Reply Queue Name used in PQ functions.
glb_message_expiry      - PQ Message Expiry (see LANDP PROGRAMMING REFERENCE for details)
glb_PQ_message_opts     - PQ Message Options (see LANDP PROGRAMMING REFERENCE for details)
glb_Qhandle             - The Queue Handle identifies the handle to use with PQ or GQ
glb_message             - The Message to be used in PQ and GQ functions.



INDIVIDUAL TESTS
----------------

The Individual Tests are a series of self-contained routines that are designed to perform a specific
task or tasks, using a combination of the API functions.  They are not as flexible as the Interactive
Test regards parameter selection, but they are more representative of the type of client functions
that would be written to used the LANDP MQ Link server. Each Test has a summary of the functions that
it is designed to perform, and has a commentary regarding the progress during the test.  The Tests use
the following parameters, which are in the #define block following these comments.

MAX_WAIT                    - MQ Timeout value (see note below)
LANDP_WAIT                  - LANDP Timeout value (see note below)
DFLT_TIMEOUT                - Default LANDP Timeout value (see note below)
LANDP_SAMPLE_USER           - The User Id use in PQ and GQ functions
LANDP_SAMPLE_QUEUE_ONE      - MQ Queue Name
LANDP_SAMPLE_QUEUE_TWO      - MQ Queue Name
LANDP_MESSAGE_ID            - The Message Id used in PQ and GQ function
LANDP_CORRELATION_ID        - The Correlation Id used in PQ and GQ functions
MQ_SERVER                   - LANDP MQ Link server Name (LANDP client definition)
SAMPLE_MESSAGE_SIZE         - Maximum Message size to be used in the tests (see note below)



NOTES
-----

1.  Both LANDP and MQ operate with seperately maintained timeout values. To avoid the situation where
    LANDP times out before MQ has completed an operation, the timeout value passed to the access_MQ
    routine (that ultimately defines the LANDP timeout) should be greater that the MQ timeout value.

2.  SAMPLE_MESSAGE_SIZE should not be greater than the maximum message size defined to the MQ Link
    Server. The maximum message size (1024 bytes by default) can be changed by a parameter passed
    on the LOADER statement for EHCMQ## (/L:lllll). See LANDP INSTALLATION AND CUSTOMIZATION for more
    details. The maximum message size that the LANDP MQSeries Link server will accept is 57000.

3.  The default value for Qhandle defines a new queue handle for each of the functions that is 
    performed, this is not be the most efficient use of resources.  It is best to reuse the same
    Qhandle for the same function. eg PQ should always use the same handle that is returned from
    the first PQ call.

-----------------------------------------------------------------------------------------------------
*/

#include "mqsample.h"

/* Insert a user ID that has authority to do MQOPEN
*/
#define  LANDP_SAMPLE_USER        ""                             /* VALID USER ID */

#define  MAX_WAIT                 100                            /* MQ TIMEOUT (SECS * 10) */
#define  LANDP_WAIT               60                             /* LANDP TIMEOUT (SECS) */
#define  DFLT_TIMEOUT             0                              /* DEFAULT LANDP TIMEOUT  */
#define  LANDP_SAMPLE_QUEUE_ONE   "LANDP.SAMPLE.QUEUE.ONE  "     /* LOCAL QUEUE NAME (ONE) */
#define  LANDP_SAMPLE_QUEUE_TWO   "LANDP.SAMPLE.QUEUE.TWO  "     /* LOCAL QUEUE NAME (TWO) */
#define  LANDP_MESSAGE_ID         "LANDP.MESSAGE.ID        "     /* MESSAGE ID */
#define  LANDP_CORRELATION_ID     "LANDP.CORRELATION.ID    "     /* CORRELATION ID */
#define  MQ_SERVER                "EHCMQ01 "                     /* LANDP MQ LINK SERVER NAME */
#define  SAMPLE_MESSAGE_SIZE      128                            /* MAXIMUM MESSAGE SIZE */

char            glb_server [8+1]                         = MQ_SERVER;
char            glb_queue_name [EHCMQLEN_QName+1]        = LANDP_SAMPLE_QUEUE_ONE;
char            glb_message_id [EHCMQLEN_MsgId+1]        = LANDP_MESSAGE_ID;
char            glb_correlation_id [EHCMQLEN_CorrelId+1] = LANDP_CORRELATION_ID;
char            glb_user_id [EHCMQLEN_UserId+1]          = LANDP_SAMPLE_USER;
long            glb_wait_interval                        = MAX_WAIT;
unsigned short  glb_GQ_message_opts                      = EHCGQ_WAIT+EHCGQ_FAIL_IF_QUIESCING;
unsigned short  glb_BR_message_opts                      = EHCGQ_BROWSE_NEXT;
unsigned short  glb_message_type                         = EHCMT_DATAGRAM;
char            glb_reply_queue_name [EHCMQLEN_QName+1]  = LANDP_SAMPLE_QUEUE_TWO;
long            glb_message_expiry                       = EHCMQ_NO_TIMEOUT;
unsigned short  glb_PQ_message_opts                      = EHCPQ_FAIL_IF_QUIESCING;
unsigned short  glb_Qhandle                              = NEW_QHANDLE;
char            glb_code[12+1]                           = "0  OK       ";
char            glb_message [SAMPLE_MESSAGE_SIZE+1]      = "A Sample Message";

union mtype {
   unsigned short  uss;
   char            c [2];
} mtype;


union code {
   unsigned short  uss;
   char            c [2];
} code;

/*------------------------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------------------------*/
void main (int  argc,
           char *argv [])
{
   unsigned long ret_code;
   int   user_test;

   /* Define the application name */
   memcpy (cprb.ehcresource_origin, "MQSAMPLE", 8);

   ret_code = init_Landp ();

   do {
      user_test = test_type ();

      switch (user_test) {
         case 1 :
            setup_parameters ();
            break;
         case 2 :
            begin_transaction ();
            break;
         case 3 :
            put_to_q ();
            break;
         case 4 :
            get_from_q ();
            break;
         case 5 :
            browse_from_q ();
            break;
         case 6 :
            rollback_transaction ();
            break;
         case 7 :
            commit_transaction ();
            break;
         case 8 :
            end_transaction ();
            break;
         case 9 :
            close_session ();
            break;

         default:
            ret_code = exit_Landp ();
            exit (0);
      }
      fflush(stdout);

   } while (TRUE);
}


/*------------------------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------------------------*/
int test_type (
             void
            )
{
   char test_no [10];
   int  max_choice;
   int  choice = 0;
   int  count;

   do {
      count = 0;
      printf ("\n\n");
      printf ("    %2d. Setup Parameters \n", ++count);
      printf ("    %2d. Start a Transaction \n", ++count);
      printf ("    %2d. Put a Message to a Queue \n", ++count);
      printf ("    %2d. Get a Message from a Queue \n", ++count);
      printf ("    %2d. Browse a Message from a Queue \n", ++count);
      printf ("    %2d. Rollback the Current Transaction \n", ++count);
      printf ("    %2d. Commit the Current Transaction \n", ++count);
      printf ("    %2d. End the Current Transaction \n", ++count);
      printf ("    %2d. Close the Session \n", ++count);
      printf ("    %2d. End \n\n", ++count);
      printf ("      Enter Required Test No. [%d] -> ", count);
      fflush(stdout);
      max_choice = count;
      gets (test_no);
      if (sscanf (test_no, "%d", &choice) == EOF)  choice = count;
   } while ((choice < 1) || (choice > max_choice));

   return (choice);
}


/*------------------------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------------------------*/
long init_Landp (
                 void
                )
{
  long landprc=OK;

  cprb.ehcfunct      = IN;
  cprb.ehctimeout    = 0;
  cprb.ehcqparml     = 26;
  cprb.ehcqdatal     = 0;
  cprb.ehcrparml     = 26;
  cprb.ehcrdatal     = 0;
  cprb.ehcqparmad    = req_parm_area;
  cprb.ehcrparmad    = rply_parm_area;
  cprb.ehcservnamlen = 8;
  memcpy (cprb.ehcserver, "SPV     ", 8);
  landprc = RMTREQ (&cprb, EHC_RESERVED);
  if (cprb.ehcretcode != 0){
      landprc = cprb.ehcretcode;
      printf("IN gave ehcretcode x%08x\n",cprb.ehcretcode);
  }
  return (landprc);
}


long exit_Landp (
                 void
                )
{
  long landprc=OK;

  cprb.ehcfunct      = EJ;
  cprb.ehctimeout    = 0;
  cprb.ehcqparml     = 0;
  cprb.ehcrparml     = 0;
  cprb.ehcqdatal     = 0;
  cprb.ehcrdatal     = 0;
  cprb.ehcservnamlen = 8;
  memcpy (cprb.ehcserver, "SPV     ", 8);
  memcpy (cprb.ehcresource_origin, "        ", 8);
  landprc = RMTREQ (&cprb, EHC_RESERVED);
  if (cprb.ehcretcode != 0){
      landprc = cprb.ehcretcode;
      printf("EJ gave ehcretcode x%08x\n",cprb.ehcretcode);
  }
  return (landprc);
}


long control_MQ (
                 unsigned short fun_code,
                 MQREASON       *reason_code_MQ
                )
{

/*  Functions  BT, CP, CS, ET, RB  */

   long landprc;

   *reason_code_MQ=0;

   cprb.ehcfunct      = fun_code;
   cprb.ehctimeout    = 0;
   cprb.ehcqparml     = 0;
   cprb.ehcqdatal     = 0;
   cprb.ehcrparml     = 4;
   cprb.ehcrdatal     = 0;
   cprb.ehcrparmad    = reason_code_MQ;
   cprb.ehcservnamlen = 8;
   memcpy (cprb.ehcserver, glb_server, 8);
   landprc = RMTREQ (&cprb, EHC_RESERVED);
   if (cprb.ehcretcode != 0)  landprc = cprb.ehcretcode;
   return (landprc);
}


long closeq_MQ (
                unsigned short handle_MQ,
                MQREASON       *reason_code_MQ
               )
{

/*  Function  CQ  */

   long landprc;
   *reason_code_MQ=0;

   cprb.ehcfunct      = EHC_CloseQ;
   cprb.ehctimeout    = 0;
   cprb.ehcqparml     = 2;
   cprb.ehcqdatal     = 0;
   cprb.ehcrparml     = 4;
   cprb.ehcrdatal     = 0;
   cprb.ehcqparmad    = &handle_MQ;
   cprb.ehcrparmad    = reason_code_MQ;
   cprb.ehcservnamlen = 8;
   memcpy (cprb.ehcserver, glb_server, 8);
   landprc = RMTREQ (&cprb, EHC_RESERVED);
   if (cprb.ehcretcode != 0)  landprc = cprb.ehcretcode;
   return (landprc);
}


long access_MQ (
                unsigned short fun_code,
                void           *input_parm_MQ,
                void           *output_parm_MQ,
                char           *message_MQ,
                unsigned short *message_len_MQ,
                long            max_timeout            /* 1/10 seconds */
               )
{

/* Functions GQ, PQ, P1  */

   long landprc;

   cprb.ehcfunct      = fun_code;

   if (fun_code == EHC_GetQ) {
      /* BEWARE : LANDP can timeout before MQGET does, make sure that
         the LANDP timeout is kept in step with the GQ wait interval.
      */
      if (max_timeout < 0)                             /* wait forever */
          cprb.ehctimeout = 0xFFFF;
      else if (max_timeout == DFLT_TIMEOUT)            /* use the default */
          cprb.ehctimeout = DFLT_TIMEOUT;
      else if(((max_timeout/10)+LANDP_WAIT) > 0xFFFF)  /* check for overflow */
          cprb.ehctimeout = 0x7FFF;
      else
          cprb.ehctimeout = (unsigned short)((max_timeout/10)+LANDP_WAIT);

      cprb.ehcqparml  = GQ_REQUEST_PARM_LEN;
      cprb.ehcqdatal  = 0;
      cprb.ehcrparml  = GQ_REPLY_PARM_LEN;
      cprb.ehcrdatal  = *message_len_MQ;
      cprb.ehcrdataad = message_MQ;
   }
   else {
      cprb.ehctimeout = (unsigned short)max_timeout/10;
      cprb.ehcqparml  = PQ_REQUEST_PARM_LEN;
      cprb.ehcqdatal  = *message_len_MQ;
      cprb.ehcrparml  = PQ_REPLY_PARM_LEN;
      cprb.ehcrdatal  = 0;
      cprb.ehcqdataad = message_MQ;
   }
   cprb.ehcqparmad    = input_parm_MQ;
   cprb.ehcrparmad    = output_parm_MQ;
   cprb.ehcservnamlen = 8;
   memcpy (cprb.ehcserver, glb_server, 8);
   landprc = RMTREQ (&cprb, EHC_RESERVED);
   if (cprb.ehcretcode != 0)  landprc = cprb.ehcretcode;
   if (fun_code == EHC_GetQ) {
      *message_len_MQ = cprb.ehcreplddlen;
   }
   return (landprc);
}
/*------------------------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------------------------*/

/* Conversion function : code2str
   NOTE this code is intel specific, and not portable.
   For format purposes buf is always given 12 characters.
*/
char * code2str(
              char *buf,
              unsigned long rc
             )
{
    code.uss=(unsigned short)rc;    /* if rc holds character values they are */
                                    /* normally in the lower order bytes.    */
    if (rc==OK)
        strcpy(buf,"0  OK       ");
    else if (isgraph(code.c[1]) && isgraph(code.c[0]))
        sprintf(buf,"x%08X %c%c",code.uss, code.c[1],code.c[0]);
    else
        sprintf(buf,"x%08X   ",rc);
    return(buf);
}

/* makeId : a function to generate a string format identifier.
   Unique enough for this application
*/
char * makeId(
             char *buf,
             char *str,
     unsigned int  len
             )
{
    unsigned int  i;
    char timeStamp[20];
    char start[10];
    time_t long_time;
    struct tm *now;

    time( &long_time );
    now=localtime( &long_time );
                          /* %j is day of the year 001-366 */
    i=strftime(timeStamp, 20,"%j_%H:%M:%S",now);

    i=len-(i+7);          /* 7  is length of "%c%c%04u_" */
    if (strlen(str) <= i)  strcpy(start,str);
    else                  strncpy(start,str,i);

    if (isalnum(cprb.ehcpc_id[0]) && isalnum(cprb.ehcpc_id[1]))
        sprintf(buf,"%s%c%c%04u_%s", start,
                                     cprb.ehcpc_id[0],cprb.ehcpc_id[1],
                                     getpid(),
                                     timeStamp);
    else
        sprintf(buf,"%sBadCPRB%s", start,timeStamp);
    return(buf);
}

/*------------------------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------------------------*/
void setup_parameters (
                          void
                         )
{
   int  count;
   int  choice;
   char test_no [10];

   do {
      count = 0;
      mtype.uss = glb_message_type;
      printf ("\n\n Parameter Summary : \n");
      printf (" %2d. MQ Link Server Name        : '%s' \n", ++count, glb_server);
      printf (" %2d. Queue Handle               :  %04x\n", ++count, glb_Qhandle);
      printf (" %2d. Queue Name                 : '%s' \n", ++count, glb_queue_name);
      printf (" %2d. Message ID                 : '%s' \n", ++count, glb_message_id);
      printf (" %2d. Correlation ID             : '%s' \n", ++count, glb_correlation_id);
      printf (" %2d. User ID                    : '%s' \n", ++count, glb_user_id);
      printf (" %2d. Get Queue Wait Interval    :  %d  \n", ++count, glb_wait_interval);
      printf (" %2d. Get Queue Message Options  :  %04x\n", ++count, glb_GQ_message_opts);
      printf (" %2d. Get Browse Message Option  :  %04x\n", ++count, glb_BR_message_opts);
      printf (" %2d. Put Queue Message Type     :  %c%c\n", ++count, mtype.c [1], mtype.c [0]);
      printf (" %2d. Put Reply Queue Name       : '%s' \n", ++count, glb_reply_queue_name);
      printf (" %2d. Put Queue Message Expiry   :  %d  \n", ++count, glb_message_expiry);
      printf (" %2d. Put Queue Message Options  :  %04x\n", ++count, glb_PQ_message_opts);
      printf (" %2d. Message                    : '%s' \n", ++count, glb_message);
      printf (" %2d. End Parameter Setup \n\n", ++count);
      printf (" Enter Parameter No. to Change [%d to End] -> ", count);
      fflush(stdout);

      gets (test_no);
      if (sscanf (test_no, "%d", &choice) == EOF)  choice = count;

      switch (choice) {
         case 1:
            printf ("\n Replace ## with a new MQ Link Server ID -> EHCMQ##\b\b");
            fflush(stdout);
            gets(test_no);
            if (strlen(test_no)==2) sprintf(glb_server,"EHCMQ%s ",test_no);
            break;
         case 2:
            printf ("\n Enter Queue Handle value (%u=NEW_QHANDLE) -> ",NEW_QHANDLE);
            fflush(stdout);
            gets (test_no);
            if (sscanf (test_no, "%x", &glb_Qhandle) == EOF)  glb_Qhandle = NEW_QHANDLE;
            break;
         case 3 :
            printf ("\n Enter New Queue Name (Max %d Chars) -> ", EHCMQLEN_QName);
            fflush(stdout);
            gets (glb_queue_name);
            break;
         case 4 :
            printf ("\n Enter New Message ID (Max %d Chars)", EHCMQLEN_MsgId);
            printf ("\n Press Enter to select NULL or a default ID -> ");
            fflush(stdout);
            gets (glb_message_id);
            if (strlen(glb_message_id)==0){
                printf ("\n Enter D for a string default, or any other key for a NULL ID -> ");
                fflush(stdout);
                gets (test_no);
                if ((test_no[0]=='D') || (test_no[0]=='d'))  makeId(glb_message_id,"MsgId",EHCMQLEN_MsgId);
                else  strcpy(glb_message_id,EHCMQCI_NONE);
            }
            break;
         case 5 :
            printf ("\n Enter New Correlation ID (Max %d Chars)", EHCMQLEN_CorrelId);
            printf ("\n Press Enter for a default or NULL ID -> ");
            fflush(stdout);
            gets (glb_correlation_id);
            if (strlen(glb_correlation_id)==0){
                printf ("\n Enter D for a string default, or any other key for a NULL ID -> ");
                fflush(stdout);
                gets (test_no);
                if ((test_no[0]=='D') || (test_no[0]=='d'))  makeId(glb_correlation_id,"Corel",EHCMQLEN_CorrelId);
                else  strcpy(glb_correlation_id,EHCMQCI_NONE);
            }
            break;
         case 6 :
            printf ("\n Enter New User ID (Max %d Chars) -> ", EHCMQLEN_UserId);
            fflush(stdout);
            gets (glb_user_id);
            break;
         case 7 :
            printf ("\n                                 EHCMQ_NO_TIMEOUT is %d",EHCMQ_NO_TIMEOUT);
            printf ("\n Enter New Get Queue Wait Interval (1/10 seconds) -> ");
            fflush(stdout);
            gets (test_no);
            if (sscanf (test_no, "%d", &glb_wait_interval) == EOF)  glb_wait_interval = MAX_WAIT;
            break;
         case 8 :
            printf ("\n Use the following bit mask format to define Options.");
            printf ("\n             EHCGQ_NONE                 0000");
            printf ("\n             EHCGQ_CONVERT              0001");
            printf ("\n             EHCGQ_FAIL_IF_QUIESCING    0010");
            printf ("\n             EHCGQ_WAIT                 0100");
            printf ("\n             EHCGQ_ACCEPT_TRUNCATED_MSG 1000");
            printf ("\n Enter New Get Queue Message Options -> ");
            fflush(stdout);
            gets (test_no);
            if (strlen(test_no)==4){
                glb_GQ_message_opts=EHCGQ_NONE;
                if (test_no[0]=='1')  glb_GQ_message_opts+=EHCGQ_ACCEPT_TRUNCATED_MSG;
                if (test_no[1]=='1')  glb_GQ_message_opts+=EHCGQ_WAIT;
                if (test_no[2]=='1')  glb_GQ_message_opts+=EHCGQ_FAIL_IF_QUIESCING;
                if (test_no[3]=='1')  glb_GQ_message_opts+=EHCGQ_CONVERT;
            }
            break;
         case 9 :
            printf ("\n Use the following bit mask format to define Options.");
            printf ("\n             EHCGQ_BROWSE_FIRST               01");
            printf ("\n             EHCGQ_BROWSE_MSG_UNDER_CURSOR    10");
            printf ("\n             EHCGQ_BROWSE_NEXT                11");
            printf ("\n Enter New Get Queue Browse Message Option -> ");
            fflush(stdout);
            gets (test_no);
            if (strlen(test_no)==2){
                glb_BR_message_opts=EHCGQ_NONE;
                if (strcmp (test_no, "01")==0) glb_BR_message_opts = EHCGQ_BROWSE_FIRST;
                if (strcmp (test_no, "10")==0) glb_BR_message_opts = EHCGQ_BROWSE_MSG_UNDER_CURSOR;
                if (strcmp (test_no, "11")==0) glb_BR_message_opts = EHCGQ_BROWSE_NEXT;
            }
            break;
         case 10 :
            printf ("\n                      EHCMT_REQUEST  RQ");
            printf ("\n                      EHCMT_REPLY    RP");
            printf ("\n                      EHCMT_DATAGRAM DG");
            printf ("\n Enter New Put Queue Message Type -> ");
            fflush(stdout);
            gets (test_no);
            if (strcmp (test_no, "DG")==0)  glb_message_type = EHCMT_DATAGRAM;
            if (strcmp (test_no, "RQ")==0)  glb_message_type = EHCMT_REQUEST;
            if (strcmp (test_no, "RP")==0)  glb_message_type = EHCMT_REPLY;
            break;
         case 11 :
            printf ("\n Enter New Reply Queue Name (Max %d Chars) -> ", EHCMQLEN_QName);
            fflush(stdout);
            gets (glb_reply_queue_name);
            break;
         case 12 :
            printf ("\n Enter New Put Queue Message Expiry (1/10 seconds) -> ");
            fflush(stdout);
            gets (test_no);
            if (sscanf (test_no, "%d", &glb_message_expiry) == EOF)  glb_message_expiry = EHCMQ_NO_TIMEOUT;
            break;
         case 13 :
            printf ("\n Use the following bit mask format to define Options.");
            printf ("\n              EHCPQ_NONE                000");
            printf ("\n              EHCPQ_NEW_MSG_ID          001");
            printf ("\n              EHCPQ_NEW_CORREL_ID       010");
            printf ("\n              EHCPQ_FAIL_IF_QUIESCING   100");
            printf ("\n Enter New Put Queue Message Options -> ");
            fflush(stdout);
            gets (test_no);
            if (strlen(test_no)==3){
                glb_PQ_message_opts=EHCPQ_NONE;
                if (test_no[0]=='1')  glb_PQ_message_opts+=EHCPQ_FAIL_IF_QUIESCING;
                if (test_no[1]=='1')  glb_PQ_message_opts+=EHCPQ_NEW_CORREL_ID;
                if (test_no[2]=='1')  glb_PQ_message_opts+=EHCPQ_NEW_MSG_ID;
            }
            break;
         case 14 :
            printf (" Enter New Message (Max %d Chars) -> ", SAMPLE_MESSAGE_SIZE);
            fflush(stdout);
            gets (glb_message);
            break;
         default :
            return;
            break;
      }

   } while (TRUE);

}
/*------------------------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------------------------*/
void begin_transaction (
                           void
                          )
{
   MQREASON  reason_code;
   long      ret_code;

   ret_code = control_MQ (EHC_BeginTran, &reason_code);
   printf ("\n\nBegin Tran   \trc=%s MQreason=%08ld \n\n", \
           code2str(glb_code,ret_code), reason_code);
   return;
}
/*------------------------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------------------------*/
void put_to_q ()
{
   long               ret_code;

   unsigned short     output_message_length;

   PQ_REQUEST_PARM_P  input_parm_put;
   PQ_REPLY_PARM_P    output_parm_put;

   input_parm_put  = (PQ_REQUEST_PARM_P) malloc (sizeof (PQ_REQUEST_PARM));
   output_parm_put = (PQ_REPLY_PARM_P) malloc (sizeof (PQ_REPLY_PARM));

   memset ((void *) input_parm_put, '\0', sizeof (PQ_REQUEST_PARM));
   memset ((void *) output_parm_put, '\0', sizeof (PQ_REPLY_PARM));

   input_parm_put->Qhandle = glb_Qhandle;
   input_parm_put->msgT    = glb_message_type;
   input_parm_put->msgExp  = glb_message_expiry;
   input_parm_put->opt     = glb_PQ_message_opts;
   output_message_length   = strlen (glb_message);

   memcpy (input_parm_put->Qname,  glb_queue_name,       strlen (glb_queue_name));
   memcpy (input_parm_put->msgId,  glb_message_id,       strlen (glb_message_id));
   memcpy (input_parm_put->correl, glb_correlation_id,   strlen (glb_correlation_id));
   memcpy (input_parm_put->user,   glb_user_id,          strlen (glb_user_id));
   memcpy (input_parm_put->replyQ, glb_reply_queue_name, strlen (glb_reply_queue_name));

   ret_code = access_MQ (EHC_PutQ, (void *) (input_parm_put), (void *) (output_parm_put),
                         glb_message, &output_message_length, DFLT_TIMEOUT);

   printf ("\n\nPut Queue    \trc=%s MQreason=%08ld  Qhandle=x%04x \n", \
           code2str(glb_code,ret_code), output_parm_put->MQreason, output_parm_put->Qhandle);
   printf ("\tmsgId  [%-24.24s]   user [%-12.12s] \n", output_parm_put->msgId, output_parm_put->user);
   printf ("\tcorrel [%-24.24s] \n", output_parm_put->correl);
   printf ("           %s \n\n", glb_message);

   free ((void *) input_parm_put);
   free ((void *) output_parm_put);

   return;
}
/*------------------------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------------------------*/
void get_from_q ()
{
   long            ret_code;
   char            input_message_area [SAMPLE_MESSAGE_SIZE+1] = "";
   unsigned short  input_message_length;
   char            user_reply;
   char            arg[10];

   GQ_REQUEST_PARM_P  input_parm_get;
   GQ_REPLY_PARM_P    output_parm_get;

   input_parm_get  = (GQ_REQUEST_PARM_P) malloc (sizeof (GQ_REQUEST_PARM));
   output_parm_get = (GQ_REPLY_PARM_P) malloc (sizeof (GQ_REPLY_PARM));

   memset ((void *) input_parm_get, '\0', sizeof (GQ_REQUEST_PARM));
   memset ((void *) output_parm_get, '\0', sizeof (GQ_REPLY_PARM));

   input_parm_get->Qhandle = glb_Qhandle;
   input_parm_get->wait    = glb_wait_interval;
   input_parm_get->opt     = glb_GQ_message_opts;
   input_message_length    = sizeof (input_message_area);

   memcpy (input_parm_get->Qname,  glb_queue_name,     strlen (glb_queue_name));
   memcpy (input_parm_get->msgId,  glb_message_id,     strlen (glb_message_id));
   memcpy (input_parm_get->correl, glb_correlation_id, strlen (glb_correlation_id));
   memcpy (input_parm_get->user,   glb_user_id,        strlen (glb_user_id));

   ret_code = access_MQ (EHC_GetQ, (void *) (input_parm_get), (void *) (output_parm_get),
                         input_message_area, &input_message_length, glb_wait_interval);

   printf ("\n\nGet Queue    \trc=%s MQreason=%08ld  Qhandle=x%04x \n", \
           code2str(glb_code,ret_code), output_parm_get->MQreason, output_parm_get->Qhandle);
   printf ("\tmsgId  [%-24.24s]   user [%-12.12s] \n", output_parm_get->msgId, output_parm_get->user);
   printf ("\tcorrel [%-24.24s] \n", output_parm_get->correl);
   printf ("           %s \n\n", input_message_area);
   fflush(stdout);

   if (output_parm_get->msgT==EHCMT_REQUEST){
       printf ("\n\n This is a Request message do you want to Reply (Y/N) ?  -> ");
       fflush(stdout);
       gets (arg);
       if (sscanf (arg, "%c", &user_reply) == EOF)  user_reply = 'N';
       if ((user_reply == 'Y')||(user_reply == 'y')) {
           glb_message_type=EHCMT_REPLY;
           memcpy(glb_queue_name, output_parm_get->replyQ,\
                 ((strlen(output_parm_get->replyQ) >= EHCMQLEN_QName) \
                  ?(EHCMQLEN_QName) :(strlen(output_parm_get->replyQ))) );
           printf("\n Queue Name is now set to the Reply Queue name and type EHCMT_REPLY.\n");
           setup_parameters ();
           printf("\n Select 'Put a Message to a Queue' to PUT your REPLY.");
           fflush(stdout);
           /* drop back to main */
       }
   }

   free ((void *) input_parm_get);
   free ((void *) output_parm_get);

   return;
}
/*------------------------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------------------------*/
void browse_from_q ()
{
   long            ret_code;
   char            user_reply;
   char            test_no [10];
   char            input_message_area [SAMPLE_MESSAGE_SIZE+1] = "";
   unsigned short  input_message_length;
   unsigned short  temp_qhandle = glb_Qhandle;

   GQ_REQUEST_PARM_P  input_parm_get;
   GQ_REPLY_PARM_P    output_parm_get;

   input_parm_get  = (GQ_REQUEST_PARM_P) malloc (sizeof (GQ_REQUEST_PARM));
   output_parm_get = (GQ_REPLY_PARM_P) malloc (sizeof (GQ_REPLY_PARM));

   memset ((void *) input_parm_get, '\0', sizeof (GQ_REQUEST_PARM));
   memset ((void *) output_parm_get, '\0', sizeof (GQ_REPLY_PARM));

   do {
      memset(input_message_area, '\0', SAMPLE_MESSAGE_SIZE+1);

      input_parm_get->Qhandle = temp_qhandle;
      input_parm_get->wait    = glb_wait_interval;
      input_parm_get->opt     = glb_GQ_message_opts+glb_BR_message_opts;
      input_message_length    = sizeof (input_message_area);

      memcpy (input_parm_get->Qname,  glb_queue_name,     strlen (glb_queue_name));
      memcpy (input_parm_get->msgId,  glb_message_id,     strlen (glb_message_id));
      memcpy (input_parm_get->correl, glb_correlation_id, strlen (glb_correlation_id));
      memcpy (input_parm_get->user,   glb_user_id,        strlen (glb_user_id));
      memset ((void *) input_message_area, '\0', sizeof (input_message_area));

      ret_code = access_MQ (EHC_GetQ, (void *) (input_parm_get), (void *) (output_parm_get),
                            input_message_area, &input_message_length, glb_wait_interval);
      printf ("\n\nGet Queue \trc=%s MQreason=%08ld  Qhandle=x%04x \n", \
              code2str(glb_code,ret_code), output_parm_get->MQreason, output_parm_get->Qhandle);

      if (output_parm_get->MQreason != 0)  break;
      if (input_parm_get->Qhandle==NEW_QHANDLE) temp_qhandle = output_parm_get->Qhandle;

      printf ("\tmsgId  [%-24.24s]   user [%-12.12s] \n", output_parm_get->msgId, output_parm_get->user);
      printf ("\tcorrel [%-24.24s] \n", output_parm_get->correl);
      printf ("           %s \n\n", input_message_area);

      printf ("\n\n Continue Browsing (Y/N) ?  -> ");
      fflush(stdout);

      gets (test_no);
      if (sscanf (test_no, "%c", &user_reply) == EOF)  user_reply = 'Y';
   } while ((user_reply == 'Y')||(user_reply == 'y')) ;

   free ((void *) input_parm_get);
   free ((void *) output_parm_get);

   return;
}
/*------------------------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------------------------*/
void rollback_transaction (
                              void
                             )
{
   MQREASON  reason_code;
   long      ret_code;

   ret_code = control_MQ (EHC_RollBack, &reason_code);
   printf ("\n\nRollback     \trc=%s MQreason=%08ld \n\n", \
           code2str(glb_code,ret_code), reason_code);
   return;
}
/*------------------------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------------------------*/
void end_transaction (
                            void
                           )
{
   MQREASON  reason_code;
   long      ret_code;

   ret_code = control_MQ (EHC_EndTran, &reason_code);
   printf ("\n\nEnd Tran     \trc=%s MQreason=%08ld \n\n", \
           code2str(glb_code,ret_code), reason_code);
   return;
}
/*------------------------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------------------------*/
void close_session (
                       void
                      )
{
   MQREASON  reason_code;
   long      ret_code;

   ret_code = control_MQ (EHC_CloseSes, &reason_code);
   printf ("\n\nClose Session\trc=%s MQreason=%08ld \n\n", \
           code2str(glb_code,ret_code), reason_code);
   return;
}

/*------------------------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------------------------*/
void commit_transaction (
                            void
                           )
{
   MQREASON  reason_code;
   long      ret_code;

   ret_code = control_MQ (EHC_CheckPoint, &reason_code);
   printf ("\n\nCommit Tran  \trc=%s MQreason=%08ld \n\n", \
           code2str(glb_code,ret_code), reason_code);
   return;
}

